import { Mapper } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { Type } from './core/domain/username.types';
import { UsernameEntity } from './core/domain/username.entity';
import { UsernameModel } from './infrastructure/username.repository';
import { UsernameResponseDto } from './interface/dtos/username.response.dto';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class UsernameMapper
  implements
    Mapper<UsernameEntity, UsernameModel, UsernameModel, UsernameResponseDto>
{
  toPersistence = (entity: UsernameEntity): UsernameModel => {
    const copy = entity.getProps();
    const record: UsernameModel = {
      authUuid: copy.userId,
      username: copy.name,
      type: copy.type,
      createdAt: copy.createdAt,
      updatedAt: copy.updatedAt,
    };
    return record;
  };

  toDomain = (record: UsernameModel): UsernameEntity => {
    const entity = new UsernameEntity({
      id: record.username,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        name: record.username,
        type: Type[record.type],
        userId: record.authUuid,
      },
    });
    return entity;
  };

  toResponse = (entity: UsernameEntity): UsernameResponseDto => {
    const response = new UsernameResponseDto(entity);
    return response;
  };
}
